Attribute VB_Name = "MurlTimer"
Option Explicit

' URLLabel written by David Crowell (davidc@qtm.net)
' See http://www.qtm.net/~davidc for updates
' Released into the public domain, since I believe that trying to sell such
' a simple piece of work would be silly.  Use this code at your own risk,
' modify as you wish.
'
' This module is necessary to receive the callback from the API timer.  I
' could have put a timer control on the usercontrol, but that would require
' a seperate timer for each instance of the control.  This method only requires
' one timer to run, and this module's TimerProc calls each control's TimerProc.
'
' If you don't have a need for a text color change or underline with the mouse
' over the label, then you may remove this module, and make the appropriate
' changes is the usercontrol code.  You will also need to copy the ShellExecute
' declare into the usercontrol code.

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' needed API declares

' GetCursorPos and GetWindowRect are to determine whether or not the
' cursor is over the URLLabel
Public Declare Function GetCursorPos Lib "user32" ( _
                                                    lpPoint As POINTAPI _
                                                    ) As Long

Public Declare Function GetWindowRect Lib "user32" ( _
                                                    ByVal hwnd As Long, _
                                                    lpRect As RECT _
                                                    ) As Long
                                                    
' ShellExecute is used to open the default browser to the desired URL
Public Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" ( _
                                                    ByVal hwnd As Long, _
                                                    ByVal lpOperation As String, _
                                                    ByVal lpFile As String, _
                                                    ByVal lpParameters As String, _
                                                    ByVal lpDirectory As String, _
                                                    ByVal nShowCmd As Long _
                                                    ) As Long

' SetTimer and KillTimer are used to create a timer that calls a procedure
' occasionly to check the cursor position
Private Declare Function SetTimer Lib "user32" ( _
                                                    ByVal hwnd As Long, _
                                                    ByVal nIDEvent As Long, _
                                                    ByVal uElapse As Long, _
                                                    ByVal lpTimerFunc As Long _
                                                    ) As Long

Private Declare Function KillTimer Lib "user32" ( _
                                                    ByVal hwnd As Long, _
                                                    ByVal nIDEvent As Long _
                                                    ) As Long
                                                    
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Type RECT
        Left As Long
        Top As Long
        Right As Long
        Bottom As Long
End Type

Public Type POINTAPI
        x As Long
        y As Long
End Type

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' private module variables
Private Running As Boolean
Private Clients() As UrlLabel
Private numClients As Long
Private UsageCount As Long
Private hTmr As Long

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Function AddClient(Client As UrlLabel) As Long
' Add another control to be notifed of timer events,
' returns an ID number to the calling control
    Dim i As Long
    Dim ClNum As Long
    
    ' look for an empty slot
    For i = 1 To numClients
        If Clients(i) Is Nothing Then
            Set Clients(i) = Client
            ClNum = i
            Exit For
        End If
    Next
    If ClNum Then
        AddClient = ClNum
        UpdateUsage 1
        Exit Function
    End If
    ' add another slot if none empty
    numClients = numClients + 1
    ReDim Preserve Clients(1 To numClients)
    Set Clients(numClients) = Client
    AddClient = numClients
    UpdateUsage 1
End Function

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Sub RemoveClient(ID As Long)
' remove a client by ID number
    If ID <= numClients And ID > 0 Then
        UpdateUsage -1
        Set Clients(ID) = Nothing
    End If
End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Sub UpdateUsage(val As Long)
    UsageCount = UsageCount + val
    
    If UsageCount > 0 Then
        'enable timer
        If Not Running Then
            hTmr = SetTimer(0, 0, 100, AddressOf TimerProc)
            Running = True
        End If
    Else
        'disable timer
        If Running Then
            Call KillTimer(0, hTmr)
            Running = False
        End If
    End If
End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Private Sub TimerProc(ByVal hwnd As Long, ByVal uMsg As Long, ByVal idEvent As Long, ByVal dwTime As Long)
    Dim i As Long
    ' check for active clients
    For i = 1 To numClients
        If Not Clients(i) Is Nothing Then
            ' call those that are registered
            Clients(i).TimerProc
        End If
    Next i
End Sub
